#ifndef BUSSIM_DHID_H_INCLUDED
#define BUSSIM_DHID_H_INCLUDED


class DHID_ID {
    // This class is used to store a unique identifier to all positions where a
    // vehicle in public transport is planned to stop at or depart from.
    //
    // IFOPT-Standard <CountryCode>:<AdminArea>:<StopPlace>:<Level>:<Quay>
    // DHID (VDV432)  <CountryCode>:<AdminArea>:<StopPlace>:<Level>:<Quay>:<StopPosition>
    //
    // BUSSIM_STOP-Objekte werden durch <CountryCode>:<AdminArea>:<StopPlace> identifiziert
    // BUSSIM_STOP-OBjekte werden durch <CountryCode>:<AdminArea>:<StopPlace>:<Level>:<Quay> oder
    // <CountryCode>:<AdminArea>:<StopPlace>:<Level>:<Quay>:<StopPosition> gekennzeichnet, je nach Datenquelle
    // (in Deutschland: DHID-Standard)
    //

    friend class BUSSIM_DHID_StopPlaceList;
    friend class BUSSIM_DHID_StopPlace;
    friend class BUSSIM_DHID_QuayList;
    friend class BUSSIM_DHID_Quay;
    friend class DHID_LIST;
    friend class NetexLine;
    std::string ORIGINAL_CODE;      // hier wird einfach der unzerlegte Schlssel abgelegt
    std::string CountryCode;        // DHID: Landeskennzeichen (LK)
    std::string AdminArea;          // DHID: Kreisschssel (KS)
    std::string StopPlace;          // DHID: Lokale Haltestellen ID
    std::string Level;              // DHID: Bereichs-ID, z.B. Ebene in einer UBahn
    std::string Quay;               // DHID: Mast-ID
    std::string StopPosition;       // DHID: Halteposition
    std::string OriginalName;       // the original name printed in maps, etc.
    int is_set;                     // needed to decide if this instance has been set or not
    double longitude;
    double latitude;

public:
    DHID_ID();              // the standard constructor
    DHID_ID(std::string _ORIGINAL_CODE, std::string _ORIGINALNAME, double _LONGITUDE, double _LATITUDE);
    ~DHID_ID();             // the standard destructor
    void from_string(std::string _STRTOCONVERT); // splits up _STRTOCONVERT into attributes
};

class BUSSIM_DHID_StopPlace {

    friend class BUSSIM_DHID_StopPlaceList;
    friend class DHID_LIST;
    // this object template is used to store a DHID_StopPlace-instance
    class DHID_ID ID;
    std::string NETEX_Id;
    class BUSSIM_DHID_StopPlace *ptr2next;
    class BUSSIM_DHID_StopPlace *ptr2prev;

    public:
    BUSSIM_DHID_StopPlace();
    BUSSIM_DHID_StopPlace(class DHID_ID _ID);
    BUSSIM_DHID_StopPlace(class DHID_ID _ID, std::string _NETEX_Id);
    ~BUSSIM_DHID_StopPlace();

    void print(void);
};

class BUSSIM_DHID_StopPlaceList {

    // this object hosts a double-linked list of StopPlace-Objects
    long StopPlaces;        // the number of stored StopPlace-Items
    class BUSSIM_DHID_StopPlace *ref2first;  // pointer to first stored StopPlace-Item
    class BUSSIM_DHID_StopPlace *ref2last;  // pointer to first stored StopPlace-Item

public:
    BUSSIM_DHID_StopPlaceList();    // the standard constructor
    ~BUSSIM_DHID_StopPlaceList();   // the standard destructor

    void Append_StopPlace(class BUSSIM_DHID_StopPlace *_StopPlace);
    void print(void);
    long get_stopplaces(void);
    class BUSSIM_DHID_StopPlace *already_stored_level3(class DHID_ID _ID);
};

class BUSSIM_DHID_Quay {

    friend class BUSSIM_DHID_QuayList;
    friend class DHID_LIST;
    // this object template is used to store a DHID_Quay-instance
    class DHID_ID ID;
    std::string NETEX_Id;
    int SERVED;
    class BUSSIM_DHID_Quay *ptr2next;
    class BUSSIM_DHID_Quay *ptr2prev;
public:
    BUSSIM_DHID_Quay();
    BUSSIM_DHID_Quay(class DHID_ID _ID);
    BUSSIM_DHID_Quay(class DHID_ID _ID, std::string _NETEX_Id);
    ~BUSSIM_DHID_Quay();
    void print(void);
};

class BUSSIM_DHID_QuayList {

    friend class DHID_LIST;
    // this object hosts a double-linked list of Quay-Objects
    long Quays;        // the number of stored Quay-Items
    class BUSSIM_DHID_Quay *ref2first;  // pointer to first stored StopPlace-Item
    class BUSSIM_DHID_Quay *ref2last;  // pointer to first stored StopPlace-Item

public:
    BUSSIM_DHID_QuayList();    // the standard constructor
    ~BUSSIM_DHID_QuayList();   // the standard destructor

    void Append_Quay(class BUSSIM_DHID_Quay *_Quay);
    void print(void);
    long get_quays(void);
    class BUSSIM_DHID_Quay *already_stored_level3(class DHID_ID _ID);
    class BUSSIM_DHID_Quay *already_stored_level5(class DHID_ID _ID);
    class BUSSIM_DHID_Quay *already_stored_netexid(std::string _NETEX_Id);
};

class DHID_LIST {
    // this is the core object template
    // added in version 3.1
    //
    // this object mainly stores a list of double-linked DHID-StopPlace-Objects as well as a list of double-linked
    // DHID-Quay-Objects. Management methods are also part of this object.

    friend class NetexLineMaster_List;
    friend class NetexLineMaster;

    class BUSSIM_DHID_StopPlaceList *StopPlaceList;
    class BUSSIM_DHID_QuayList *QuayList;
    class NetexLineMaster_List *MasterLIneList;
    class NetexLine_List *LineList;

    std::string ScenarioName;

    int FILES;      // number of netex files im scenario
    std::string *FILENAME;  // array of netex file names

public:
    DHID_LIST();
    ~DHID_LIST();
    void print(void);
    void analyze_from_netex_xml(std::string _FILE);
    void analyze_from_netex_xml_lines(std::string _FILE);
    void export_quay_list_to_xml(void);
    void export_line_list_to_xml(void);

    void import_from_xml(std::string _FILE);
    void import_netexfile_list(std::string _FILE);
    int get_files(void);
    std::string get_file_name_at_list_pos(int _ListPos);
    class BUSSIM_DHID_Quay *ptr2_nearest_quay(double _LON, double _LAT);
    void set_name(std::string _ScenarioName);
    std::string get_name(void);
};


#endif // BUSSIM_DHID_H_INCLUDED
